unit FaxServer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  WinFax, FileCtrl, ShellAPI;

const
   WM_FAX_EVENT = WM_USER + 5432;

type
   TFaxDocumentsList = class;
   TFaxDocument = class;
   TFaxPortsList = class;
   TFaxCoverPage = class;

   TStatusMessageSeverity = (smsError, smsDebug, smsStatus, smsDirectAccessLog, smsWaveError);

   TFaxEvent = procedure(Sender : TObject; FaxDoc: TFaxDocument) of object;
   TFaxProblemEvent = procedure(Sender : TObject; EventID: DWORD; FaxDoc: TFaxDocument) of object;
   TStatusMessage = procedure (Severity: TStatusMessageSeverity; ErrorID: DWORD; MsgStr: string) of object;

   TFaxService = class(TComponent)
   private
      { Private declarations }
      FaxHandle: THANDLE;
      bTerminate: boolean;
      EventHandle: HWND;
      hCompletionPort: THANDLE;

      FArchiveDirectoryIn : string;
      FArchiveDirectoryOut : string;

      FOnFaxIdle : TNotifyEvent;
      FOnFaxCompleted : TFaxEvent;
      FOnFaxClose : TNotifyEvent;
      FOnFaxReceiving : TFaxEvent;
      FOnFaxSending : TFaxEvent;
      FOnFaxProblem : TFaxProblemEvent;
      FOnStatusMessage : TStatusMessage;
      FOnFaxAnswered : TFaxEvent;
      FOnFaxRinging : TNotifyEvent;

      procedure EventHandler(var Msg: TMessage);
      procedure SetStatusMessage(Severity: TStatusMessageSeverity; ErrorID: Cardinal; MsgStr: string);
      procedure CheckConfiguration;
      procedure SetArchiveDirectoryIn(Value: String);
      procedure SetArchiveDirectoryOut(Value: String);
//      procedure StartChangeWaiting(Dir: string);
//      procedure StopChangeWaiting;
   protected
      { Protected declarations }
   public
      { Public declarations }
      ActiveDeviceID: DWORD;
      FaxPortsList: TFaxPortsList;
      FaxDocList: TFaxDocumentsList;
      SenderName: string;           //pointer to sender's name
      SenderCompany: string;        //pointer to sender's company
      SenderDept: string;           //pointer to sender's department
      BillingCode: string;          //pointer to billing code

      { fax configuration }
      Retries : DWORD;
      RetryDelay : DWORD;
      DirtyDays : DWORD;
      Branding : boolean;
      UseDeviceTsid : boolean;
      ServerCp : boolean;
      PauseServerQueue : boolean;
      StartCheapTime : TFAX_TIME;
      StopCheapTime : TFAX_TIME;
      ArchiveOutgoingFaxes : boolean;

      constructor Create(AOwner: TComponent); override;
      procedure Connect(FaxServer: string);
      procedure Disconnect;
      function CreateFaxDocument: TFaxDocument;
      procedure StopEventWaiting;
      procedure StartEventWaiting;
      procedure EnableRoutingExtension(Enable: boolean);
      function IsWaiting:boolean;
      function GetMsgStr(EventId: DWORD): string;
   published
      { Published declarations }
      property ArchiveDirIn : string read FArchiveDirectoryIn write SetArchiveDirectoryIn;
      property ArchiveDirOut : string read FArchiveDirectoryOut write SetArchiveDirectoryOut;

      property OnFaxIdle : TNotifyEvent read FOnFaxIdle write FOnFaxIdle;
      property OnFaxCompleted : TFaxEvent read FOnFaxCompleted write FOnFaxCompleted;
      property OnFaxClose : TNotifyEvent read FOnFaxClose write FOnFaxClose;
      property OnFaxReceiving : TFaxEvent read FOnFaxReceiving write FOnFaxReceiving;
      property OnFaxSending : TFaxEvent read FOnFaxSending write FOnFaxSending;
      property OnFaxProblem : TFaxProblemEvent read FOnFaxProblem write FOnFaxProblem;
      property OnFaxAnswered: TFaxEvent read FOnFaxAnswered write FOnFaxAnswered;
      property OnFaxRinging: TNotifyEvent read FOnFaxRinging write FOnFaxRinging;
      property OnStatusMessage: TStatusMessage read FOnStatusMessage write FOnStatusMessage;
   end;

   TFaxDocumentsList = class(TCollection)
   public
      FaxServer: TFaxService;
      constructor Create(AFaxServer: TFaxService);
      procedure LoadFromStream(Stream: TStream);
      procedure SaveToStream(Stream: TStream);
      function GetDocByJobID(JobID: DWORD):TFaxDocument;
      procedure SaveToFile(FileName: string);
      procedure LoadFromFile(FileName: string);
   end;

   TFaxDocument = class(TCollectionItem)
   private
      procedure CopyJobDataToDoc(pJobEntry: PFAX_JOB_ENTRY);
   public
      FaxJobId: DWORD;
      FaxServer: TFaxService;

      RecipientNumber: string;      //pointer to recipient's fax number
      RecipientName: string;        //pointer to recipient's name
      Tsid: string;                 //pointer to transmitting station identifier
      SenderName: string;           //pointer to sender's name
      SenderCompany: string;        //pointer to sender's company
      SenderDept: string;           //pointer to sender's department
      BillingCode: string;          //pointer to billing code
      ScheduleAction: TFAX_ENUM_JOB_SEND_ATTRIBUTES;
                                    //job scheduling action code
      ScheduleTime: TDateTime;      //time to send fax
      DeliveryReportType: TFAX_ENUM_DELIVERY_REPORT_TYPES;
                                    //e-mail delivery report type
      DeliveryReportAddress: string;//pointer to e-mail address
      DocumentName: string;         //pointer to document name to display
      FileName: string;
      TIFFFileName: string;

      JobType: DWORD;               //job type (send/receive)
      UserName: string;             //pointer to user who submitted job

      QueueStatus: DWORD;           //set of job status bit flags
      Status: DWORD;                //status code for device
      Size: DWORD;                  //size of document, in bytes
      PageCount: DWORD;             //total pages in transmission
      SendReceiveTime: TDateTime;   //send and receive time
      CallerId: String;             //CallerId

      constructor Create(FaxDocList: TCollection); override;
      procedure Send(CoverPage: TFaxCoverPage);
      procedure SendSheduled(ScheduleAction: TFAX_ENUM_JOB_SEND_ATTRIBUTES; ScheduleTime: TDateTime; CoverPage: TFaxCoverPage);
      procedure LoadFromStream(Stream: TStream);
      procedure SaveToStream(Stream: TStream);
      procedure FindTIFF;
      procedure ShowFax(ParentWindow: HWND);
   end;

   TFaxPortsList = class(TCollection)
   public
      FaxServer: TFaxService;
      constructor Create(AFaxServer: TFaxService);
      procedure Refresh;
   end;

   TFaxPort = class(TCollectionItem)
   public
      DeviceId: DWORD;      // line identifier of the device
      State: DWORD;         // fax device status
      Flags: DWORD;         // set of fax port capability bit flags
      Rings: DWORD;         // rings before answer
      Priority: DWORD;      // priority for device selection
      DeviceName: string;   // pointer to device name
      Tsid: string;         // pointer to
                            // transmitting station identifier
      Csid: string;         // pointer to called station identifier

      constructor Create(FaxPortsList: TFaxPortsList);
   end;

   TFaxCoverPage = Class(TComponent)
   private
      //
      // General information
      //
      FCoverPageName: string;      //cover page document name
      FUseServerCoverPage: boolean;//local/server file flag
      //
      // Recipient information
      //
      FRecName: string;            //recipient's name
      FRecFaxNumber: string;       //fax number
      FRecCompany: string;         //company name
      FRecStreetAddress: string;   //street address
      FRecCity: string;            //city
      FRecState: string;           //state
      FRecZip: string;             //zip code
      FRecCountry: string;         //country
      FRecTitle: string;           //title
      FRecDepartment: string;      //department
      FRecOfficeLocation: string;  //office location
      FRecHomePhone: string;       //home number
      FRecOfficePhone: string;     //office number
      //
      // Sender information
      //
      FSdrName: string;            //sender's name
      FSdrFaxNumber: string;       //fax number
      FSdrCompany: string;         //company name
      FSdrAddress: string;         //address
      FSdrTitle: string;           //title
      FSdrDepartment: string;      //department
      FSdrOfficeLocation: string;  //office location
      FSdrHomePhone: string;       //home number
      FSdrOfficePhone: string;     //office number
      //
      // Miscellaneous information
      //
      FNote: string;               //note text
      FSubject: string;            //subject line text
      FTimeSent: TDateTime;        //time fax was sent
      FPageCount: DWORD;           //total pages in transmission

      CoverPage: TFAX_COVERPAGE_INFO;
      procedure RefreshStruct;
   published
      property CoverPageName: string read FCoverPageName write FCoverPageName;
      property UseServerCoverPage: boolean read FUseServerCoverPage write FUseServerCoverPage;
      property RecName: string read FRecName write FRecName;
      property RecFaxNumber: string read FRecFaxNumber write FRecFaxNumber;
      property RecCompany: string read FRecCompany write FRecCompany;
      property RecStreetAddress: string read FRecStreetAddress write FRecStreetAddress;
      property RecCity: string read FRecCity write FRecCity;
      property RecState: string read FRecState write FRecState;
      property RecZip: string read FRecZip write FRecZip;
      property RecCountry: string read FRecCountry write FRecCountry;
      property RecTitle: string read FRecTitle write FRecTitle;
      property RecDepartment: string read FRecDepartment write FRecDepartment;
      property RecOfficeLocation: string read FRecOfficeLocation write FRecOfficeLocation;
      property RecHomePhone: string read FRecHomePhone write FRecHomePhone;
      property RecOfficePhone: string read FRecOfficePhone write FRecOfficePhone;
      property SdrName: string read FSdrName write FSdrName;
      property SdrFaxNumber: string read FSdrFaxNumber write FSdrFaxNumber;
      property SdrCompany: string read FSdrCompany write FSdrCompany;
      property SdrAddress: string read FSdrAddress write FSdrAddress;
      property SdrTitle: string read FSdrTitle write FSdrTitle;
      property SdrDepartment: string read FSdrDepartment write FSdrDepartment;
      property SdrOfficeLocation: string read FSdrOfficeLocation write FSdrOfficeLocation;
      property SdrHomePhone: string read FSdrHomePhone write FSdrHomePhone;
      property SdrOfficePhone: string read FSdrOfficePhone write FSdrOfficePhone;
      property Note: string read FNote write FNote;
      property Subject: string read FSubject write FSubject;
      property TimeSent: TDateTime read FTimeSent write FTimeSent;
      property PageCount: DWORD read FPageCount write FPageCount;
   end;

   TEventWaitingThread = class(TThread)
   private
      bTerminate : boolean;
      FaxServer : TFaxService;
   protected
      procedure Execute; override;
   public
   end;
(*
   TChangeWaitingThread = class(TThread)
   private
      { Private declarations }
   protected
      procedure Execute; override;
   public
      Dir: string;
      hTerminateEvent: THANDLE;
   end;
  *)
   procedure CheckFax(Error: Boolean);
   procedure Register;

   procedure WriteString(s: string; Stream: TStream);
   procedure WriteInt(i: integer; Stream: TStream);
   procedure ReadString(var s: string; Stream: TStream);
   procedure ReadInt(var i: integer; Stream: TStream);

implementation
